@@ -1,4 +1,4 @@
-#' Convert flow data to SpatialLinesDataFrame
+#' Extract coordinates from OD data
 #'
 #' @section Details:
 #' Origin-destination ('OD') flow data is often provided
@@ -14,15 +14,60 @@
 #' @param zones A SpatialPolygonsDataFrame or SpatialPointsDataFrame
 #' representing origins and destinations of travel flows.
 #' @references
-#' Rae, A. (2009). From spatial interaction data to spatial interaction information? Geovisualisation and spatial structures of migration from the 2001 UK census. Computers, Environment and Urban Systems, 33(3). doi:10.1016/j.compenvurbsys.2009.01.007
+#' Rae, A. (2009). From spatial interaction data to spatial interaction information?
+#' Geovisualisation and spatial structures of migration from the 2001 UK census.
+#'  Computers, Environment and Urban Systems, 33(3). doi:10.1016/j.compenvurbsys.2009.01.007
+#' @export
+#' @examples \dontrun{
+#' od2odf(flow, zones)
+#' }
+od2odf <- function(flow, zones){
+
+  coords = dplyr::data_frame(code = as.character(zones[[1]]),
+                      fx = coordinates(zones)[,1], fy = coordinates(zones)[,2])
+  flowcode = dplyr::data_frame(code_o = as.character(flow[[1]]), code_d = as.character(flow[[2]]))
+  odf = dplyr::left_join(flowcode, coords, by = c("code_o" = "code"))
+  coords = dplyr::rename_(coords, tx = quote(fx), ty = quote(fy))
+  odf = dplyr::left_join(odf, coords, by = c("code_d" = "code"))
+
+  data.frame(odf) # return data.frame as more compatible with spatial data
+
+}
+#' Convert flow data to SpatialLinesDataFrame
+#'
+#' @section Details:
+#' Origin-destination ('OD') flow data is often provided
+#' in the form of 1 line per flow with zone codes of origin and destination
+#' centroids. This can be tricky to plot and link-up with geographical data.
+#' This function makes the task easier.
+#' \code{\link{od2line2}} is a faster implementation
+#' (around 6 times faster on large datasets)
+#' that returns a \code{SpatialLines} object (omitting the data).
+#'
+#' @param flow A data frame representing the flow between two points
+#' or zones. The first two columns of this data frame should correspond
+#' to the first column of the data in the zones. Thus in \code{\link{cents}},
+#' the first column is geo_code. This corresponds to the first two columns
+#' of \code{\link{flow}}.
+#' @param zones A SpatialPolygonsDataFrame or SpatialPointsDataFrame
+#' representing origins and destinations of travel flows.
 #' @export
 #' @examples \dontrun{
 #' data(flow) # load data frame of od flows between zones
 #' data(cents) # load centroids data
 #' newflowlines <- od2line(flow = flow, zones = cents)
-#' plot(cents)
-#' lines(newflowlines)
+#' newflowlines2 <- od2line2(flow = flow, zones = cents)
+#' sp::plot(cents)
+#' lines(newflowlines, lwd = 3)
+#' lines(newflowlines2, col = "white")
+#' nfl_sldf <- SpatialLinesDataFrame(newflowlines, flow, match.ID = FALSE)
+#' identical(nfl_sldf, newflowlines)
 #' }
+#' @name od2line
+NULL
+
+#' @rdname od2line
+#' @export
 od2line <- function(flow, zones){
   l <- vector("list", nrow(flow))
   for(i in 1:nrow(flow)){
@@ -37,46 +82,69 @@ od2line <- function(flow, zones){
     l[[i]] <- sp::Lines(list(sp::Line(rbind(x, y))), as.character(i))
   }
   l <- sp::SpatialLines(l)
-  l <- sp::SpatialLinesDataFrame(l, data = flow, match.ID = F)
+  l <- sp::SpatialLinesDataFrame(l, data = flow, match.ID = FALSE)
   sp::proj4string(l) <- sp::proj4string(zones)
   l
 }
 
-#' Convert straight SpatialLinesDataFrame to a data.frame with from and to coords
+#' @rdname od2line
+#' @export
+od2line2 <- function(flow, zones){
+
+  odf = od2odf(flow, zones)
+  l <- vector("list", nrow(odf))
+  for(i in 1:nrow(odf)){
+    l[[i]] <- sp::Lines(list(sp::Line(rbind(c(odf$fx[i], odf$fy[i]), c(odf$tx[i], odf$ty[i])))), as.character(i))
+  }
+  l <- sp::SpatialLines(l)
+}
+
+#' Convert SpatialLinesDataFrame objects to a data.frame with from and to coords
 #'
+#' This function returns a data frame with fx and fy and tx and ty variables
+#' representing the beginning and end points of spatial line features respectively.
 #'
 #' @param l A SpatialLinesDataFrame
 #' @export
 #' @examples
-#' \dontrun{
-#' data(flowlines) # load demo flowlines dataset
-#' ldf <- line2df(flowlines)
-#' }
+#' line2df(flowlines[5,]) # beginning and end of a single straight line
+#' line2df(flowlines) # on multiple lines
+#' line2df(routes_fast[5:6,]) # beginning and end of routes
 line2df <- function(l){
-  l_list <- lapply(slot(l, "lines"), function(x) lapply(slot(x, "Lines"),
-  function(y) slot(y, "coords")))
-  from_list <- lapply(l@lines, function(x) x@Lines[[1]]@coords[1,])
-  to_list <- lapply(l@lines, function(x) x@Lines[[1]]@coords[2,])
-  from_mat <- do.call(rbind, from_list)
-  to_mat <- do.call(rbind, to_list)
-
-  output <- as.data.frame(cbind(from_mat, to_mat))
-  names(output) <- c("fx", "fy", "tx", "ty")
-
-  output
-
+  ldf_geom = raster::geom(l)
+  dplyr::group_by_(dplyr::as_data_frame(ldf_geom), 'object') %>%
+    summarise_(fx = quote(first(x)), fy = quote(first(y)), tx = quote(last(x)), ty = quote(last(y)))
 }
 
-#' Convert a SpatialLinesDataFrame to points at the origin and destination
+#' Convert a SpatialLinesDataFrame to points
+#'
+#' The number of points will be double the number of lines with \code{line2points}.
+#' A closely related function, \code{line2pointsn} returns all the points that were line vertices.
+#' The points corresponding with a given line, \code{i}, will be \code{(2*i):((2*i)+1)}.
 #'
 #' @param l A SpatialLinesDataFrame
 #' @export
 #' @examples
 #' data(routes_fast)
-#' lpoints <- line2points(routes_fast[2,]) # for a single line
+#' lpoints <- line2pointsn(routes_fast[2,]) # for a single line
+#' lpoints2 = line2points(routes_fast[2,])
+#' plot(lpoints)
+#' plot(lpoints2)
+#' lpoints = line2pointsn(routes_fast) # for many lines
+#' plot(lpoints)
 #' data(flowlines) # load demo flowlines dataset
 #' lpoints <- line2points(flowlines) # for many lines
+#' sp::proj4string(lpoints) # maintains CRS info
 #' plot(lpoints) # note overlapping points
+#' i = 3
+#' j = (2*i):((2*i)+1)
+#' plot(flowlines[i,])
+#' plot(lpoints[j,], add = TRUE)
+#' @name line2points
+NULL
+
+#' @rdname line2points
+#' @export
 line2points <- function(l){
   for(i in 1:length(l)){
     l1 <- l[i,]
@@ -91,6 +159,15 @@ line2points <- function(l){
   }
   out
 }
+
+#' @rdname line2points
+#' @export
+line2pointsn <- function(l){
+  spdf = raster::geom(l)
+  p = sp::SpatialPoints(coords = spdf[,c("x", "y")])
+  raster::crs(p) = raster::crs(l)
+  p
+}
 #' Convert straight SpatialLinesDataFrame from flow data into routes
 #'
 #' @section Details:
@@ -110,8 +187,8 @@ line2points <- function(l){
 #' plot(flowlines)
 #' rf <- line2route(l = flowlines, "route_cyclestreet", plan = "fastest")
 #' rq <- line2route(l = flowlines, plan = "quietest", silent = TRUE)
-#' plot(rf, col = "red", add = T)
-#' plot(rq, col = "green", add = T)
+#' plot(rf, col = "red", add = TRUE
+#' plot(rq, col = "green", add = TRUE
 #' # Plot for a single line to compare 'fastest' and 'quietest' route
 #' n = 21
 #' plot(flowlines[n,])
@@ -156,10 +233,13 @@ line2route <- function(l, route_fun = "route_cyclestreet", n_print = 10, ...){
 #' @export
 #' @examples
 #' df <- points2odf(cents)
-#'
+#' cents_centroids <- rgeos::gCentroid(cents, byid = TRUE)
+#' df2 <- points2odf(cents_centroids)
 points2odf <- function(p){
   if(grepl(pattern = "DataFrame", class(p))){
     geo_code <- p@data[,1]
+  } else if(is(p, "SpatialPoints")){
+    geo_code <- 1:length(p)
   } else {
     geo_code <- p[,1]
   }
@@ -214,3 +294,55 @@ update_line_geometry <- function(l, nl){
   }
   l
 }
+
+#' Quickly calculate Euclidean distances of od pairs
+#'
+#' It is common to want to know the Euclidean distance between origins and destinations
+#' in OD data. You can calculate this by first converting OD data to SpatialLines data,
+#' e.g. with \code{\link{od2line}}. However this can be slow and overkill if you just
+#' want to know the distance. This function is a few orders of magnitude faster.
+#'
+#' Note: this function assumes that the zones or centroids in \code{cents} have a geographic
+#' (lat/lon) CRS.
+#'
+#' @inheritParams od2line
+#' @export
+#' @examples
+#' data(flow)
+#' data(cents)
+#' od_dist(flow, cents)
+od_dist <- function(flow, zones){
+  omatch = match(flow[[1]], zones@data[[1]])
+  dmatch = match(flow[[2]], zones@data[[1]])
+  cents_o = zones@coords[omatch,]
+  cents_d = zones@coords[dmatch,]
+  geosphere::distHaversine(p1 = cents_o, p2 = cents_d)
+}
+
+#' Convert a series of points, or a matrix of coordinates, into a line
+#'
+#' This is a simple wrapper around \code{\link{spLines}} that makes the creation of
+#' \code{SpatialLines} objects easy and intuitive
+#'
+#' @param p A SpatialPoints obect or matrix representing the coordinates of points.
+#' @export
+#' @examples
+#' p = matrix(1:4, ncol = 2)
+#' l = points2line(p)
+#' plot(l)
+#' l = points2line(cents)
+#' plot(l)
+#' p = line2points(routes_fast)
+#' l = points2line(p)
+#' plot(l)
+points2line = function(p){
+  if(is(p, "SpatialPoints")){
+    p_proj = sp::proj4string(p)
+    p = sp::coordinates(p)
+  } else {
+    p_proj = NA
+  }
+  l = raster::spLines(p)
+  raster::crs(l) = p_proj
+  l
+}
