@@ -75,10 +75,13 @@ lineLabels <- function(sldf, attrib){
 #' and converts these into a single route network.
 #'
 #' @param sldf A SpatialLinesDataFrame with overlapping elements
-#' @param attrib A text string corresponding to the variable in \code{sldf$} on
-#' which the function will operate.
-#' @param fun The function used to aggregate the grouped values (default: sum)
+#' @param attrib A character vector corresponding to the variables in
+#' \code{sldf$} on which the function(s) will operate.
+#' @param fun The function(s) used to aggregate the grouped values (default: sum).
+#' If length of \code{fun} is smaller than \code{attrib} then the functions are
+#' repeated for subsequent attributes.
 #' @param na.zero Sets whether aggregated values with a value of zero are removed.
+#' @param byvars Character vector containing the column names to use for grouping
 #'
 #' @author Barry Rowlingson
 #' @references
@@ -110,26 +113,96 @@ lineLabels <- function(sldf, attrib){
 #' flowlines_1way <- maptools::spRbind(flowlines_sub, flowlines_2way)
 #' overlaps <- over()
 #' nrow(overlaps)
+#' routes_fast$group = rep(1:3, length.out = nrow(routes_fast))
+#' rnet_grouped = overline(routes_fast, attrib = "length", byvars = "group")
+#' plot(rnet_grouped, col = rnet_grouped$group, lwd =
+#'   rnet_grouped$length / mean(rnet_grouped$length) * 3)
 #' }
-overline <- function(sldf, attrib, fun = sum, na.zero = FALSE){
-  ## simplify down to SpatialLines
-  sl = as(sldf, "SpatialLines")
-  ## get the line sections that make the network
-  slu = gsection(sl)
-  ## overlay network with routes
-  overs = sp::over(slu, sl, returnList=TRUE)
-  ## overlay is true if end points overlay, so filter them out:
-  overs = lapply(1:length(overs), function(islu){
-    Filter(function(isl){
-      islines(sl[isl,],slu[islu,])
-    }, overs[[islu]])
-  })
-  ## now aggregate the required attribibute using fun():
-  aggs = sapply(overs, function(os){fun(sldf[[attrib]][os])})
+overline <- function(sldf, attrib, fun = sum, na.zero = FALSE, byvars = NA){
 
-  ## make a SLDF with the named attribibute:
-  sldf = sp::SpatialLinesDataFrame(slu, data.frame(Z=aggs))
-  names(sldf) = attrib
+  fun <- c(fun)
+  if (length(fun) < length(attrib)) {
+    fun <- rep(c(fun),length.out=length(attrib))
+  }
+
+  if (is.na(byvars[1]) == TRUE) {
+    ## simplify down to SpatialLines
+    sl = as(sldf, "SpatialLines")
+    ## get the line sections that make the network
+    slu = gsection(sl)
+    ## overlay network with routes
+    overs = sp::over(slu, sl, returnList=TRUE)
+    ## overlay is true if end points overlay, so filter them out:
+    overs = lapply(1:length(overs), function(islu){
+      Filter(function(isl){
+        islines(sl[isl,],slu[islu,])
+      }, overs[[islu]])
+    })
+    ## now aggregate the required attribibute using fun():
+    #aggs = sapply(overs, function(os){fun(sldf[[attrib]][os])})
+    aggs <- setNames(
+      as.data.frame(
+        lapply(1:length(attrib),
+               function(y, overs, attribs, aggfuns){
+                 sapply(overs, function(os,attrib,fun2){
+                   fun2(sldf[[attrib]][os])},
+                   attrib=attribs[y],
+                   fun2=aggfuns[[y]])
+                 },
+               overs,
+               attrib,
+               fun)),
+      attrib)
+
+    ## make a SLDF with the named attribibute:
+    sldf = sp::SpatialLinesDataFrame(slu, aggs)
+    #names(sldf) = attrib
+  } else {
+
+    splitlines <- lapply(
+      split(sldf, sldf@data[,byvars]),
+      function(x, attrib, gvar){
+        groupingcat <- unname(unlist(unique(x@data[,gvar])))
+        sl = as(x, "SpatialLines")
+        slu = gsection(sl)
+        overs = sp::over(slu, sl, returnList = TRUE)
+        overs = lapply(1:length(overs), function(islu) {
+          Filter(function(isl){islines(sl[isl,],slu[islu,])}, overs[[islu]])
+        })
+        #aggs = sapply(overs, function(os){fun(x[[attrib]][os])})
+        aggs <- setNames(
+          as.data.frame(
+            lapply(1:length(attrib),
+                   function(y, overs, attribs, aggfuns){
+                     sapply(overs, function(os,attrib,fun2){
+                       fun2(x[[attrib]][os])},
+                       attrib=attribs[y],
+                       fun2=aggfuns[[y]])
+                   },
+                   overs,
+                   attrib,
+                   fun)
+            ),
+          attrib)
+        sldf = sp::SpatialLinesDataFrame(slu, cbind(aggs,as.data.frame(matrix(groupingcat,nrow=1))))
+        names(sldf) = c(attrib,gvar)
+        sldf <- spChFIDs(sldf, paste(paste(groupingcat,collapse='.'),row.names(sldf@data),sep='.'))
+        sldf
+      },
+      attrib,
+      c(byvars)
+    )
+
+    splitlinesdf <- data.frame(data.table::rbindlist(lapply(splitlines, function(x){x@data})))
+    row.names(splitlinesdf) <- unname(unlist(lapply(splitlines, function(x){row.names(x@data)})))
+
+    sldf <- SpatialLinesDataFrame(
+      SpatialLines(unlist(lapply(splitlines, function(x){x@lines}), recursive = FALSE),
+                   proj4string = splitlines[[1]]@proj4string),
+      splitlinesdf
+    )
+
+  }
 
   ## remove lines with attribute values of zero
   if(na.zero == TRUE){
@@ -172,35 +245,26 @@ overline <- function(sldf, attrib, fun = sum, na.zero = FALSE){
 #' sum(singlelines$All)
 #' nrow(singlelines)
 onewaygeo <- function(x, attrib){
-  geq <- rgeos::gEquals(x, x, byid = T)
+  geq <- rgeos::gEquals(x, x, byid = TRUE) | rgeos::gEqualsExact(x, x, byid = TRUE)
   sel1 <- !duplicated(geq) # repeated rows
-  sel2 <- rowSums(geq) > 1 # all features with an overlap
-  sel3 <- rgeos::gLength(x, byid = TRUE) > 0 # remove points
-  sel4 <- sel1 & sel2 & sel3
-  singlelines <- x[sel4,]
-  otherlines <- x[!sel4, ] # the lines that are duplicated
+  singlelines <- x[sel1,]
 
-  for(i in 1:nrow(singlelines)){
-    # select matching line
-    l2 <- which(rgeos::gEquals(singlelines[i, ], x, byid = TRUE))[2]
+  singlelines@data[,attrib] <- (matrix(
+    unlist(
+      lapply(
+        apply(geq, 1, function(x){
+          which(x == TRUE)
+        }),
+        function(y,x) {
+          colSums(x[y,3:14]@data)
+        }, x)),
+    nrow=49,
+    byrow=TRUE))[sel1,]
 
-    # only perform aggregation on flows that have a return flow
-    if(length(l2) > 0){
-      if(class(attrib) == "character"){
-        # aggregate the data for reverse flows
-        singlelines[[attrib]][i] <- singlelines[[attrib]][i] + x[[attrib]][l2]
-      } else {
-        # aggregate the data for reverse flows
-        singlelines@data[i, attrib] <- singlelines@data[i, attrib] +
-          as.numeric(x@data[l2, attrib])
-      }
-    }
-
-  }
-  singlelines
+  return(singlelines)
 }
 
-#' Aggregate flows so they become non-directional (by id - the faster way)
+#' Aggregate ods so they become non-directional, e.g. by summing travel in both directions.
 #'
 #' Flow data often contains movement in two directions: from point A to point B
 #' and then from B to A. This can be problematic for transport planning, because
@@ -216,56 +280,57 @@ onewaygeo <- function(x, attrib){
 #' potentially halving the number of lines objects and reducing the number
 #' of overlapping lines to zero.
 #'
-#' @param x A SpatialLinesDataFrame
-#' @param attrib A text string containing the name of the line's attribute to
-#' aggregate or a numeric vector of the columns to be aggregated
-#' @param id1 A text string referring to the name of the variable containing the unique id of the origin
-#' @param id2 A text string referring to the name of the variable containing the unique id of the destination
+#' @param x A data frame, representing an OD matrix
+#' @param attrib A vector of column numbers or names
+#' for deciding which attribute(s) of class numeric to
+#' aggregate
+#' @param id1 Optional (it is assumed to be the first column)
+#' text string referring to the name of the variable containing
+#' the unique id of the origin
+#' @param id2 Optional (it is assumed to be the second column)
+#' text string referring to the name of the variable containing
+#' the unique id of the destination
 #'
-#' @return \code{onewayid} outputs a SpatialLinesDataFrame with single lines
-#' and user-selected attribute values that have been aggregated. Only lines
-#' with a distance (i.e. not intra-zone flows) are included.
+#' @return \code{onewayid} outputs a data.frame with rows containing
+#' results for the user-selected attribute values that have been aggregated.
 #' @export
 #' @examples
-#' data("flowlines")
-#' id1 <- names(flowlines)[1]
-#' id2 <- names(flowlines)[2]
-#' plot(flowlines)
-#' singlelines <- onewayid(flowlines, attrib = 3:14, id1, id2)
-#' lines(singlelines) # check we've got the right lines
-#' sum(singlelines$All)
-#' nrow(singlelines)
-#' sl2 <- onewaygeo(flowlines, attrib = 3:14)
+#' data("flow")
+#' flow_oneway = onewayid(flow, attrib = 3)
+#' nrow(flow_oneway) < nrow(flow) # result has fewer rows
+#' sum(flow$All) == sum(flow_oneway$All) # but the same total flow
+#' # using names instead of index for attribute
+#' onewayid(flow, attrib = "All")
+#' # using many attributes to aggregate
+#' attrib = which(vapply(flow, is.numeric, TRUE))
+#' flow_oneway = onewayid(flow, attrib = attrib)
+#' colSums(flow_oneway[attrib]) == colSums(flow[attrib]) # test if the colSums are equal
 #' # Demonstrate the results from onewayid and onewaygeo are identical
-#` identical(singlelines, sl2)
+#' flow_oneway_geo = onewaygeo(flowlines, attrib = attrib)
+#' plot(flow_oneway$All, flow_oneway_geo$All)
 onewayid <- function(x, attrib, id1 = names(x)[1], id2 = names(x)[2]){
-  ids <- cbind(x[[id1]], x[[id2]])
-  idsort <- t(apply(ids, 1, sort))
-  # duplicate pairs - see http://stackoverflow.com/questions/9028369/
-  sel <- !duplicated(idsort) & rgeos::gLength(x, byid = TRUE) > 0
-
-  singlelines <- x[sel,]
-  if(sum(sel) != nrow(x)) otherlines <- x[!sel, ] # the lines that are duplicated
-
-  for(i in 1:nrow(singlelines)){
-    # select matching lines
-    idp1 <- paste0(x[[id1]], x[[id2]])
-    idp2 <- paste0(x[[id2]], x[[id1]])
-    l2 <- which(idp2 %in% idp1[sel][i])
-
-    # only perform aggregation on flows that have a return flow
-    if(length(l2) > 0){
-      if(class(attrib) == "character"){
-        # aggregate the data for reverse flows
-        singlelines[[attrib]][i] <- singlelines[[attrib]][i] + x[[attrib]][l2]
-      } else {
-        # aggregate the data for reverse flows
-        singlelines@data[i, attrib] <- singlelines@data[i, attrib] +
-          as.numeric(x@data[l2, attrib])
-      }
-    }
 
+  if(is.numeric(attrib)){
+    attrib_names = names(x)[attrib]
+  } else {
+    attrib_names = attrib
+    attrib = which(names(x) %in% attrib)
   }
-  singlelines
+
+  x_oneway <- x %>%
+    dplyr::mutate_(stplanr.id1 = id1,
+                   stplanr.id2 = id2,
+                   stplanr.key = ~paste(pmin(stplanr.id1, stplanr.id2), pmax(stplanr.id1, stplanr.id2))) %>%
+    dplyr::group_by_(quote(stplanr.key)) %>%
+    dplyr::mutate(is_two_way = ifelse(n() > 1, TRUE, FALSE)) %>%
+    dplyr::mutate_each("sum", attrib) %>%
+    dplyr::summarise_each_(funs("stplanr.first"),c(id1, id2, attrib, ~is_two_way)) %>%
+    dplyr::select_(quote(-stplanr.key))
+
+  return(x_oneway)
+
 }
 
+stplanr.first <- function(...) {
+  dplyr::first(...)
+}
\ No newline at end of file
